//#include "stdafx.h"

#include "RNP2to191.h"

namespace rnp2to191 {

	inline double sqr( double v ){
			return  v * v;
	}

    //        Standard Normal Distribution
	//        Rejection polar method for normal variate
	double RNP2to191::normal(){
		double v1, v2, w;
		do {
    		v1 = 2.0 * uni() - 1.0;
			v2 = 2.0 * uni() - 1.0;
			w  = sqr(v1) + sqr(v2);
		} while (((w >= 1.0) || (w <= 0.0)));

		double c = Math::Sqrt(-2.0 * Math::Log(w)/w);

		return c * v1;
	}


    //    Normal Distribution of mean m  and standard deviation s
	double RNP2to191::normalMS(double m, double s){
			double v = s * normal() + m;
			return  v;
	}


    //    A Pair of Independent Random Numbers of 
	//      the Standard Normal Distribution 
	void  RNP2to191::normalPair( double & n1, double & n2 ){
			double v1, v2, w, c;
			do {
				  v1 = 2.0 * uni() - 1.0;
				  v2 = 2.0 * uni() - 1.0;
				  w  = sqr(v1) + sqr(v2);
			} while (((w >= 1.0) || (w <= 0.0)));

			c = Math::Sqrt(-2.0 * Math::Log(w) / w);

			n1 = c * v1;
			n2 = c * v2;
	}

	//		A Pair of Random Numbers of
	//		correlation coefficient r
	void RNP2to191::normalPair_r( double & n1, double & n2, double r ){
			double z1, z2;
			normalPair( z1, z2 );
			n1 = z1;
			n2 = r * z1 + Math::Sqrt(1 - r * r) * z2;
	}

	
	//
	//			J. E. Gentle (2003)
	//			"Random number generation and Monte Carlo methods, 2nd ed."
	//			Springer, Chapter 5.
	//
	double RNP2to191::gamma( double a, double b ){
		if ((a <= 0.0) || (b <= 0.0)){
			throw gcnew Exception("Invalid parameter value error...a = " + a.ToString() + "   b = " + b.ToString());
		}

		if (a > 1.0){
			while(true){
				double u1 = uni(), u2 = uni();
				double v = (a - 1.0 / (6 * a)) * u1 / ((a - 1.0) * u2);
				if (((2.0 * (u2 - 1.0) / (a - 1.0)) + v + 1.0 / v) <= 2.0){
					return b * (a - 1.0) * v;
				}else{
					if (((2.0 * Math::Log(u2)) / (a - 1.0) - Math::Log(v) + v) <= 1.0){
						return b * (a - 1.0) * v;
					}
				}
			}
		}else if (a < 1.0){
			double t = 0.07 + 0.75 * Math::Sqrt(1.0 - a);
			double tb = 1.0 + Math::Exp(-t) * a / t;
			while(true){
				double u1 = uni(), u2 = uni(), v = tb * u1;
				if (v <= 1.0){
					double x = t * Math::Pow( v, 1.0/a );
					if (u2 <= ((2.0 - x) / (2.0 + x))){
						return b * x;
					}else{
						if (u2 <= Math::Exp(-x)) return b * x;
					}
				}else{
					double x = -Math::Log(t * (tb - v) / a);
					double y = x / t;
					if ((u2 * (a + (y * (1.0 - a)))) <= 1.0){
						return b * x;
					}else{
						if (u2 <= Math::Pow( y, a - 1.0 )){
							return b * x;
						}
					}
				}
			}
		}else{
			return -b * Math::Log(uni());
		}
	}

	//
	//			N. L. Johnson, S. Kotz and N. Balakrishnan (1995)
	//			"Continuous univariate distributions, Vol. 2, 2nd ed.
	//			John Wiley & Sons, Inc., Chapter 25
	//	
	double RNP2to191::beta( double p, double q){
        double W1 = gamma( p, 1.0 ), W2 = gamma( q, 1.0 );
		return W1 / (W1 + W2);
	}


}
